import * as aws_lambda from 'aws-lambda';
import { APIGatewayProxyEvent, APIGatewayProxyResult, Context } from 'aws-lambda';
import * as type_fest from 'type-fest';
import { ObjectSchema } from 'joi';
import middy from '@middy/core';

declare class HttpError extends Error {
    name: string;
    statusCode: number;
    constructor(messageOrError: string | Error, statusCode: number);
}

declare class ServiceError extends Error {
    name: string;
    code: number;
    statusCode: number;
    constructor(messageOrError: string | Error, code?: number, statusCode?: number);
}

declare class BadRequestError extends HttpError {
    name: string;
    constructor(message?: string);
}

declare class ForbiddenError extends HttpError {
    name: string;
    constructor(message?: string);
}

declare class NotFoundError extends HttpError {
    name: string;
    constructor(message?: string);
}

type ApiGatewayHandlerFn = (event: APIGatewayProxyEvent, context: Context) => Promise<APIGatewayProxyResult>;
interface MiddyfyOptions<THandler> {
    handler: THandler;
    eventSchema?: ObjectSchema;
}
declare const middyfyAPIGateway: (options: MiddyfyOptions<ApiGatewayHandlerFn>) => middy.MiddyfiedHandler<Omit<APIGatewayProxyEvent, "body"> & {
    body: type_fest.JsonValue;
} & APIGatewayProxyEvent & {
    multiValueQueryStringParameters: aws_lambda.APIGatewayProxyEventMultiValueQueryStringParameters;
    pathParameters: aws_lambda.APIGatewayProxyEventPathParameters;
    queryStringParameters: aws_lambda.APIGatewayProxyEventQueryStringParameters;
}, APIGatewayProxyResult, Error, Context>;

export { BadRequestError, ForbiddenError, HttpError, NotFoundError, ServiceError, middyfyAPIGateway };
